#!/usr/bin/python3

import os, sys, struct, shutil
from sys import argv
from stat import *

# no emulator-specific headers are used, Goomba will parse ROM headers in concatenated data
# use this script for Goomba, Goomba Color, and Jagoo

def readfile(name):
	try:
		fd = open(name, "rb")
		contents = fd.read()
		fd.close()
	except IOError:
		print("Error reading", name)
		sys.exit(1)
	return contents

def writefile(name, contents):
	try:
		fd = open(name, "wb")
		fd.write(contents)
		fd.close()
	except IOError:
		print("Error writing", name)
		sys.exit(1)

if __name__ == "__main__":
	if len(argv) < 4:
		print("Usage: %s goomba.gba rom romtitle [splashscreen]")
		sys.exit(0)
	goomba = readfile(argv[1])
	rom = readfile(argv[2])
	outputfilename = os.path.splitext(argv[3])[0] + ".gba"
	outputpatchname = os.path.splitext(argv[3])[0] + ".pat"
	if len(argv) == 5:
		#splash screen is simply concatenated after emulator binary, no header needed
		writefile(outputfilename, goomba + readfile(argv[5]) + rom)
	else:
		writefile(outputfilename, goomba + rom)
	# EZ-Flash IV GSS patcher metadata to force 64KB SRAM saves (no pre-creation of .sav files is necessary for fw2.xx)
	shutil.copy2("../emupatch.pat", outputpatchname)
